###########
# COLORS  #
###########

# Fill with theme variables later

###########
# STYLES  #
###########

#### MAIN CONTENT

CONTENT_STYLE = {
    "margin-left": "18rem",
    "margin-right": "2rem",
    "padding": "2rem 1rem",
}


#### SIDEBAR

SIDEBAR_STYLE = {
    "position": "fixed",
    "top": 0,
    "left": 0,
    "bottom": 0,
    "width": "18rem",
    "padding": "2rem 1rem",
    "overflow": "auto",
    "background-color": "#f8f9fa",
}


#### TAB STYLES

TAB_HEADER_STYLE = {
    "text-align": "center"
}

#### PLOT TABLES


PLOT_GROUP_STYLE = {
    "padding": "10px",
    "display": "grid",
    
    "border-width": "2px",
    "border-style": "solid",
    "border-color": "black",
    
    "overflow": "auto"
}

PLOT_GROUP_HEADER_STYLE = {
    "text-align": "center"
}


CONTENT_GRID_STYLE = {
    "padding": "10px",
    "display": "grid",

    "border-width": "2px",
    "border-style": "solid",
    "border-color": "black",

    "overflow": "auto"
}

CONTENT_GRID_NO_BORDER_STYLE = {
    "padding": "10px",
    "display": "grid",

    "overflow": "auto"
}

CONTENT_GRID_HEADER_STYLE = {
    "text-align": "center"
}

#### PANEL STYLES

PANEL_HEADER_STYLE = {
    "text-align": "center"
}

PANEL_STYLE = {
    "padding": "10px",

    "border-width": "2px",
    "border-style": "solid",
    "border-color": "black",

    "width": "100%"
}


#### PLOT STYLES

PLOT_FRAME_STYLE = {
    "padding": "10px",
    
    "border-width": "2px",
    "border-style": "solid",
    "border-color": "black",
}

PLOT_FRAME_HEADER_STYLE = {
    "text-align": "center"
}


CHECKLIST_STYLE = {
    "padding": "10px",
    
    "border-width": "1px",
    "border-style": "solid",
    "border-color": "black",
    
    "width": "200px",
    "height": "150px"
}



#### CONTROL STYLES

CONTROL_FRAME_STYLE = {
    "padding": "10px",
    
    "border-width": "1px",
    "border-style": "solid",
    "border-color": "black",
    
    # "height": "200px",
    # "position": "relative"
}


CONTROL_STYLE = {
    "padding": "5px",

    "border-width": "1px",
    "border-style": "solid",
    "border-color": "black"
}


PLOT_CONTROL_STYLE = {
    "padding": "5px",
    
    "border-width": "1px",
    "border-style": "solid",
    "border-color": "black",
}


# Delete after refactor
CONTROL_OBJECT_STYLE = {
    "padding": "5px",

    "border-width": "1px",
    "border-style": "solid",
    "border-color": "black"
}
